/* Code written by Craig Aubuchon April 2010
   
   Goal: Create a Kernel Density plot of banks (failed and survivor) by log of assets

   Here, we'll use work.sod2006_panel.  This dataset has information on all banks in 2006, plus a dummy 
    variable fail = 1 for all banks that fail during the 2007-2010 period and missing otherwise.  I also 
    make the variable no_fail = 1 for survivors and missing otherwise.  The missing values are required 
    for the plot - otherwise there is a high density of banks with zero assets!

*/


OPTIONS LS = 75 PS = 60 NOCENTER PAGENO = 1 SYMBOLGEN MCOMPILENOTE = ALL MLOGIC;

/* A note on options: symbolgen specifies that log messages about macro variable references will be displayed
					  mcompilenote specifies that a note be completed for all macro compilations
					  mlogic specifies that messages about a macro action are printed to log during execution 
*/

LIBNAME fail            "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures";


%macro yearloop(yyyy=);
  
DATA work.panel&yyyy._graph;
  set fail.panel&yyyy;
  IF fail ~= 1 THEN no_fail = 1;
  If fail = 0 THEN fail = .;
RUN;

DATA work.panel&yyyy._graph1;
  set work.panel&yyyy._graph;
  lnasset = log(asset);
  
  lnasset_f = lnasset*fail;
  lnasset_nf = lnasset*no_fail;

RUN;

%mend; %yearloop(yyyy=1986);%yearloop(yyyy=2006);

proc means data = work.panel1986_graph1;
  class fail;
run;
proc means data = work.panel1986_graph1;
  class no_fail;
run;

/* To overlay plots, I plot the interaction of the fail/no fail dummy and log of total assets */
PROC SGPLOT data = work.panel1986_graph1;
  title1 'Fig. 4: Size Distribution of Commercial Banks, 1986:Q2';
  title2 'Kernel Density Plot of (ln) Total Assets';
  title3 'Bank Failures 1987.Q1 to 1992.Q4';
  density lnasset_f  / type = kernel legendlabel = "Failed Banks (n=884)" ;
  density lnasset_nf / type = kernel legendlabel = "Non-failing Banks (n=13205)";
  keylegend / location= outside position = bottom;
  xaxis label = "(ln) Total Assets";
run;


proc means data = work.panel2006_graph1;
  class fail;
run;
proc means data = work.panel2006_graph1;
  class no_fail;
run;

PROC SGPLOT data = work.panel2006_graph1;
  title1 'Fig. 3: Size Distribution of Banks, 2006:Q2';
  title2 'Kernel Density Plot of (ln) Total Assets';
  title3 'Bank Failures 2007.Q1 to 2010.Q1';
  density lnasset_f  / type = kernel legendlabel = "Failed Banks (n=198)" ;
  density lnasset_nf / type = kernel legendlabel = "Non-Failing Banks (n=8307)";
  keylegend / location= outside position = bottom;
  xaxis label = "(ln) Total Assets";
run;

DATA work.panel2006_graph2;
  set work.panel2006_graph1;
  IF id_rssd in (1222108) THEN DELETE;
RUN;

PROC SGPLOT data = work.panel2006_graph2;
  title1 'Size Distribution of Banks, 2006:Q2';
  title2 'Kernel Density Plot of (ln) Total Assets';
  title3 'Bank Failures 2007.Q1 to 2010.Q1, excl Washington Mutual';
  density lnasset_f  / type = kernel legendlabel = "Failed Banks (n=197)" ;
  density lnasset_nf / type = kernel legendlabel = "Non-Failing Banks (n=8307)";
  keylegend / location= outside position = bottom;
  xaxis label = "(ln) Total Assets";
run;

/* Now, make a density plot of all commercial bank failures in the 2007-10 period.  */

DATA work.panel2006_graph3;
  set fail.panel2006;
  IF fail ~= 1 THEN no_fail = 1;
  If fail = 0 THEN fail = .;

  IF fail = 1 and save = 0 THEN fail_comc = 1;
  IF fail = 1 and save = 1 THEN fail_sav = 1;

RUN;

PROC MEANS data = work.panel2006_graph3;
  VAR fail_comc fail_sav no_fail;
RUN;

DATA work.panel2006_graph4;
  set work.panel2006_graph3;
  lnasset = log(asset);
  
  lnasset_fc = lnasset*fail_comc;
  lnasset_fs = lnasset*fail_sav;
  lnasset_nf = lnasset*no_fail;

RUN;


PROC SGPLOT data = work.panel2006_graph4;
  title1 'Fig. 5: Size Distribution of Banks, 2006:Q2';
  title2 'Kernel Density Plot of (ln) Total Assets';
  title3 'Bank Failures 2007.Q1 to 2010.Q1';
  density lnasset_fc  / type = kernel legendlabel = "Failed Commercial Banks (n=162)" ;
  density lnasset_fs  / type = kernel (c=0.42) legendlabel = "Failed Savings Institutions (n=36)" ;
  density lnasset_nf /  type = kernel legendlabel = "Non-Failing Banks (n=8307)";
  keylegend / location= outside position = bottom;
  xaxis label = "(ln) Total Assets";
run;
title1;title2;title3;

/* Lets look at a list of commercial banks by size in 2006 */;

DATA fail_comc;
  set work.panel2006_graph4;
  where fail_comc = 1;
RUN;

PROC SORT data = fail_comc;
  by descending asset;
RUN;

PROC PRINT data = fail_comc;
  VAR id_rssd name asset;
  where asset > 1000000;
RUN;

/* Lets take a closer look at savings institutions */
data fail_sav;
  set work.panel2006_graph4;
  where fail_sav = 1;
  run;

proc sort data = fail_sav;
  by descending asset;
run;

proc print data = fail_sav;
  title1 'Failed Savings Institution by Size, 2006 SOD';
  var id_rssd name asset totbr;
run;
  
PROC SGPLOT data = work.fail_sav;
  title1 'Size Distribution of Savings Inst, 2006 Summary of Deposits Data';
  title2 'Kernel Density Plot of (ln) Total Assets';
  title3 'Failures 2007.Q1 to 2010.Q1';
  density lnasset_fs  / type = kernel legendlabel = "Failed Savings Institutions (n=36)" ;
  histogram lnasset_fs;
  keylegend / location= outside position = bottom;
  xaxis label = "(ln) Total Assets";
run;

data fail_sav_exclwamu;
  set work.fail_sav;
  IF rssdid = 1222108 THEN delete;
RUN;

PROC SGPLOT data = work.fail_sav_exclwamu;
  title1 'Size Distribution of Savings Inst, 2006 Summary of Deposits Data';
  title2 'Kernel Density Plot of (ln) Total Assets';
  title3 'Failures 2007.Q1 to 2010.Q1, excl WaMu';
  density lnasset_fs  / type = kernel legendlabel = "Failed Savings Institutions (n=35)" ;
  histogram lnasset_fs;
  keylegend / location= outside position = bottom;
  xaxis label = "(ln) Total Assets";
run;

/*************************************************************************************************/
* Find the largest bank by branching activity, in 2006 SOD and 2009 SOD *
/*************************************************************************************************/;

DATA branch;
  set fail.panel2006;
RUN;

PROC SORT data = work.branch;
  by descending totbr;
RUN; 

PROC PRINT data = work.branch (obs=5);
RUN;

/* Now, find branching information for countrywide */

PROC PRINT data = work.branch;
  where rssdid = 1469211;
RUN;

/* In 2009: */;


PROC SQL;

CREATE TABLE branch09 AS

SELECT sod.rssdid, sod.name, sod.cert, sod.bkbr, sod.bkmo, sod.uninumbr, sod.stnumbr, sod.cntynumb, sod.zipbr, sod.asset, sod.depdom,
       sod.hctmult, sod.hctone, sod.hctnone, sod.specgrp, sod.specdesc, 
       count(sod.rssdid)  as totbr,
       count(distinct sod.cntynumb) as tot_cntybr, 
       count(distinct sod.zipbr) as tot_zipbr, 
       count(distinct sod.stnumbr) as tot_statebr 
        	  	   
FROM raw_SOD.sodmic09 as sod

GROUP BY rssdid 
ORDER BY rssdid;

QUIT;

DATA branch09;
  SET branch09;
  WHERE bkmo = 1;
RUN;

PROC SORT data = work.branch09;
  by descending totbr;
RUN; 

PROC PRINT data = work.branch09 (obs=5);
RUN;

/* And lets look at branching activity for top 5 and countrywide in 2008 while we're here: */
PROC SQL;

CREATE TABLE branch08 AS

SELECT sod.rssdid, sod.name, sod.cert, sod.bkbr, sod.bkmo, sod.uninumbr, sod.stnumbr, sod.cntynumb, sod.zipbr, sod.asset, sod.depdom,
       sod.hctmult, sod.hctone, sod.hctnone, sod.specgrp, sod.specdesc, 
       count(sod.rssdid)  as totbr,
       count(distinct sod.cntynumb) as tot_cntybr, 
       count(distinct sod.zipbr) as tot_zipbr, 
       count(distinct sod.stnumbr) as tot_statebr 
        	  	   
FROM raw_SOD.sodmic08 as sod

GROUP BY rssdid 
ORDER BY rssdid;

QUIT;

DATA branch08;
  SET branch08;
  WHERE bkmo = 1;
RUN;

PROC SORT data = work.branch08;
  by descending totbr;
RUN; 

PROC PRINT data = work.branch08 (obs=5);
RUN;

PROC PRINT data = work.branch08;
  where rssdid = 1469211;
RUN;
