/* Program written by Craig Aubuchon in Spring 2010
    
   Goal: To pull state level data directly from haver, and create a panel of variables, ordered by state.
         
         We'll build a data set with information on house prices, unemployment and delinquency rates.

   Methodology:  I use two macro variables - one for the year/time period and one for the haver variable.

                 This allows the program to pull all of the relevant data, subset by years.  This is important
                 because we use proc means to get the max of the data set - if it isn't subset by time period, 
                 we would have max value over the entire 1985 to 2009 range.

                 I transpose the data to get state names ordered as the observations, with the variables then 
                 unemp, hpi, delinquency rates.  

                 Once I exit the macro loop, I use a merge statement to join all of the data into one large 
                 panel.  This is then used for the correlation program. 

   Data: Data comes from Haver, with the macro variable 'havername' assigned to the haver mnemonic of the 
         variable location.

   Note: This program will not run without a subscription to Haver Analytics.  I have, however, provided the 
          full data set that is created at the end.  The code in between documents transformations to the raw data. 
  

*/

LIBNAME fail "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures";

OPTIONS PS = 60 LS = 75 NOCENTER PAGENO = 1 ;
%macro varloop(libnme=,havername=,varname=,readname=,);
  %DO i = 1 %to 8;

  %let library = %SCAN(&libnme, &i);
  %let haver = %SCAN(&havername, &i);
  %let var = %SCAN(&varname,&i);
  %let name = %SCAN(&readname,&i);  

/* When pulling data directly from Haver, it is important to reassign the libname at each iteration */
LIBNAME _all_ clear;
LIBNAME &library SASEHAVR "y:\dlx\data"  FREQ = QTR START = 19820101 END = 20101231 FORCE=FREQ;

%let list = AL&var AK&var AZ&var AR&var CA&var CO&var CT&var DE&var FL&var GA&var HI&var
            ID&var IL&var IN&var IA&var KS&var KY&var LA&var ME&var MD&var MA&var MI&var 
            MN&var MS&var MO&var MT&var NE&var NV&var NH&var NJ&var NM&var NY&var NC&var 
            ND&var OH&var OK&var OR&var PA&var RI&var SC&var SD&var TN&var TX&var UT&var 
            VT&var VA&var WA&var WV&var WI&var WY&var ;

/* This step pulls the data directly in from haver */
DATA &name;
  SET &library..&haver (keep = date &list);
RUN;

PROC TRANSPOSE DATA = &name OUT = &name._panel PREFIX = &name;
  ID date;
  VAR &list;
RUN;

%END;%MEND;%varloop(libnme=hpi unemp mbamtg mbamtg mbamtg mbamtg mbamtg bankrupt,havername= regional emplr mbamtg mbamtg mbamtg mbamtg mbamtg bankrupt,varname= hpi ra L14 LCP LCS L14S LCSS bbq,readname= hpi unemp all prime sub all_n sub_n bbq);


/* Now, calculate the relevant percent change, and save (to be merged later) */

DATA hpi_correl (keep = state hpi_chg82_87 hpi_chg87_92 hpi_chg03_07 hpi_chg07_09);
  SET hpi_panel;
  state = substr(_name_,1,2); 
  hpi_chg82_87 = ((hpi1987Q1 - hpi1982Q3) / hpi1982Q3 )*100;
   label hpi_chg82_87 = "HPI Percent Change 1982Q3 to 1987Q1"; 
  hpi_chg87_92 = ((hpi1992Q4 - hpi1987Q1) / hpi1987Q1 )*100;
   label hpi_chg87_92 = "HPI Percent Change 1987Q1 to 1992Q4";
  hpi_chg03_07 = ((hpi2007Q1 - hpi2003Q3) / hpi2003Q3 )*100;
   label hpi_chg03_07 = "HPI Percent Change 2003Q3 to 2007Q1"; 
  hpi_chg07_09 = ((hpi2009Q4 - hpi2007Q1) / hpi2007Q1 )*100;
   label hpi_chg07_09 = "HPI Percent Change 2007Q1 to 2009Q4";
RUN;

proc sort data = work.hpi_correl;
  by state;
run;

/* Unemployemnt Data */

DATA unemp_correl (keep = state unemp_chg86_92 unemp_chg06_09) ;
  SET unemp_panel;
  state = substr(_name_,1,2);
  unemp_chg86_92 = unemp1992Q4 - unemp1986Q4;
   label unemp_chg86_92 = "Level Change in Unemployment Rate, (1992Q4 - 1986Q4)";
  unemp_chg06_09 = unemp2009Q4 - unemp2006Q4;
   label unemp_chg06_09 = "Level Change in Unemployment Rate, (2009Q4 - 2006Q4)";
RUN;
proc sort data = unemp_correl;
  by state;
run;

/* Delinquency Data */

DATA all_correl (keep = state all_chg86_92 all_chg06_09 all_chg07_09) ;
  SET all_panel;
  state = substr(_name_,1,2);
  all_chg86_92 = all1992Q4 - all1986Q4;
   label all_chg86_92 = "Level Change in All Loan Delinquency Rate, (1992Q4 - 1986Q4)";
  all_chg06_09 = all2009Q4 - all2006Q4;
   label all_chg06_09 = "Level Change in All Loan Delinquency Rate, (2009Q4 - 2006Q4)";
  all_chg07_09 = all2009Q4 - all2007Q1;
   label all_chg07_09 = "Level Change in All Loan Delinquency Rate, (2009Q4 - 2007Q1)";
RUN;
proc sort data = all_correl;
  by state;
run;


/* Ratio of Non-prime mortgages */

DATA sub_n_panel (keep = state sub_n2003Q3 sub_n2006Q4 sub_n2009Q4);
  SET sub_n_panel;
  state = substr(_name_,1,2);
RUN;

PROC SORT data = sub_n_panel;
  by state;
RUN;

DATA all_n_panel (keep = state all_n2003Q3 all_n2006Q4 all_n2009Q4);
  SET all_n_panel;
  state = substr(_name_,1,2);
RUN;

PROC SORT data = all_n_panel;
  by state;
RUN;

DATA num_correl;
  MERGE all_n_panel sub_n_panel;
  BY state;
  subratio03 = (sub_n2003Q3 / all_n2003Q3)*100 ;
   label subratio03 = "Percent of Subprime mortgages to all Mortgages, 2003Q3";
  subratio06 = (sub_n2006Q4 / all_n2006Q4)*100 ;
   label subratio06 = "Percent of Subprime mortgages to all Mortgages, 2006Q4";
  subratio09 = (sub_n2009Q4 / all_n2009Q4)*100 ;
   label subratio09 = "Percent of Subprime mortgages to all Mortgages, 2009Q4";
  sub_chg03_06 = subratio06 - subratio03;
   label sub_chg03_06 = "Level increase in percent of subprime mortgages, 03Q3 to 06Q4";
  sub_chg06_09 = subratio09 - subratio06;
   label sub_chg06_09 = "Level increase in percent of subprime mortgages, 06Q4 to 09Q4";
RUN;

proc sort data = num_correl;
  by state;
run;


/* Import Annual Data from Haver - Gross State Product and Per Capita Income */

%macro varloop(libnme=,havername=,varname=,readname=,);
  %DO i = 1 %to 4;

  %let library  = %SCAN(&libnme, &i);
  %let haver    = %SCAN(&havername, &i);
  %let var      = %SCAN(&varname,&i);
  %let name     = %SCAN(&readname,&i);  

LIBNAME _all_ clear;
LIBNAME &library SASEHAVR "y:\dlx\data"  FREQ = yearly START = 19820101 END = 20090101;

%let list = AL&var AK&var AZ&var AR&var CA&var CO&var CT&var DE&var FL&var GA&var HI&var
            ID&var IL&var IN&var IA&var KS&var KY&var LA&var ME&var MD&var MA&var MI&var 
            MN&var MS&var MO&var MT&var NE&var NV&var NH&var NJ&var NM&var NY&var NC&var 
            ND&var OH&var OK&var OR&var PA&var RI&var SC&var SD&var TN&var TX&var UT&var 
            VT&var VA&var WA&var WV&var WI&var WY&var ;

/* This step pulls the data directly in from haver */
DATA &name;
  SET &library..&haver (keep = date &list);
RUN;

PROC TRANSPOSE DATA = &name OUT = &name._panel PREFIX = &name;
  ID date;
  VAR &list;
RUN;

DATA &name._panel (drop = _name_);
  SET &name._panel;
  state = substr(_name_,1,2);
RUN;

PROC SORT data = &name._panel;
  BY state;
RUN;

%END;%MEND;%varloop(libnme=pci gdp loan loan,havername= pir gsp fdicr fdicr,varname= ypph to kes jes,readname= pci gdp cloan sloan);

/* Per Capita Income Data */

DATA pci_correl (keep = state pci_chg86_92 pci_chg06_09) ;
  SET pci_panel;
  pci_chg86_92 = ((pci1992 - pci1986) / pci1986)*100;
   label pci_chg86_92 = "Percent Change in Real Per Capita Income, 1986 to 1992";
  IF state ~= 'WY' THEN 
   pci_chg06_09 = ((pci2009 - pci2006) / pci2006)*100;
    label pci_chg06_09 = "Percent Change in Real Per Capita Income, 2006 to 2009";
   ELSE if state = 'WY' THEN
   pci_chg06_09 = ((pci2008 - pci2006) / pci2006)*100;
RUN;

/* Gross State Product Data */

DATA gdp_correl (keep = state gdp_chg86_92 gdp_chg06_08) ;
  SET gdp_panel;
  gdp_chg86_92 = ((gdp1992 - gdp1986) / gdp1986)*100;
   label gdp_chg86_92 = "Percent Change in Gross State Product, 1986 to 1992";
  gdp_chg06_08 = ((gdp2008 - gdp2006) / gdp2006)*100;
   label gdp_chg06_08 = "Percent Change in Gross State Product, 2006 to 2008";
RUN;

/* Gross Loans Secured by Real Estate  */
/* We need to add commercial and savings institutions together for the current period */

DATA loan_correl (keep = state cloan1982 cloan1986 cloan2002 cloan2006 sloan2002 sloan2006 loan_chg82_86 loan_chg02_06 tloan2002 tloan2006);
  MERGE cloan_panel sloan_panel;
  BY state;
  loan_chg82_86 = ((cloan1986 - cloan1982) / cloan1982)*100;
   label loan_chg82_86 = "Percent Change in Gross Loans Secured by Real Estate, 1982 to 1986, Commercial Banks";
  tloan2002 = sloan2002 + cloan2002;
   label tloan2002 = "Total loans (from commercial and savings) secured by real estate, 2002";
  tloan2006 = sloan2006 + cloan2006;
   label tloan2006 = "Total loans (from commercial and savings) secured by real estate, 2006";
  loan_chg02_06 = ((tloan2006 - tloan2002) / tloan2002)*100;
   label loan_chg02_06 = "Percent Change in Gross Loans Secured by Real Estate, 2002 to 2006, Commercial and Savings";
RUN;

/* I need to do some work to transform bankruptcy data (stored in the loop above as bbq).
    We have raw number of bankruptcies - what we really want is a ratio of bankruptcies per 1000 firms.
    First, we'll import data for the number of firms from the BLS.  Then we'll create a sum total of 
    bankruptcies per year, then we'll make the ratio.   */


PROC IMPORT OUT= WORK.firmnum 
            DATAFILE= "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\number of establishments.xlsx" 
            DBMS=EXCEL REPLACE;
     RANGE="Data$"; 
     GETNAMES=YES;
     MIXED=NO;
     SCANTEXT=YES;
     USEDATE=YES;
     SCANTIME=YES;
RUN;


DATA work.firmnum;
  set work.firmnum;
  firms_2006 = firms_2006 / 1000;
  label firms_2006 = "number of firms (thous) in 4Q";
  firms_2008 = firms_2008 / 1000;
  label firms_2008  = "number of firms (thous) in 4Q";  
run;

DATA work.bbq_panel2 (keep = state bbq2007 bbq2009);
  set work.bbq_panel;
  bbq2007 = bbq2007Q1 + bbq2007Q2 + bbq2007Q3 + bbq2007Q4;
  bbq2009 = bbq2009Q1 + bbq2009Q2 + bbq2009Q3 + bbq2009Q4;
  label bbq2007 = "Number of business bankruptcies";
  label bbq2009 = "Number of business bankruptcies";
  state = substr(_name_,1,2);
run;

proc sort data = work.bbq_panel2;
 by state;
run;

proc sort data = work.firmnum;
  by state;
run;

data work.bbq_ratio;
  MERGE work.firmnum work.bbq_panel2;
  by state;
RUN;

/* Bankruptcy Ratio Data */
data work.bbq_correl (keep = state bbq2007_ratio bbq2009_ratio bbq_chg07_09);
  set work.bbq_ratio;
  bbq2007_ratio = bbq2007 / firms_2006;
  label bbq2007_ratio = "yr2007 Bankruptcies per 1000 firms (4Q.06)"; 
  bbq2009_ratio = bbq2009 / firms_2008;
  label bbq2009_ratio = "yr2009 Bankruptcies per 1000 firms (4Q.08)";
  bbq_chg07_09 = bbq2009_ratio - bbq2007_ratio;
  label bbq_chg07_09 = "Level Change in Bankruptcy ratios 07 to 09";
run;
proc sort data = bbq_correl;
 by state;
run;

/* Now we need to add branching restriction data; lets import branching restriction data from 
    Rice and Strahan (2008-054) 'Does Credit Supply Affect Small Firm Finance?' */

PROC IMPORT OUT= WORK.Branch_restrict06 
            DATAFILE= "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\state_interstate_branching_laws.xls" 
            DBMS=EXCEL REPLACE;
     RANGE="Sheet2$"; 
     GETNAMES=YES;
     MIXED=NO;
     SCANTEXT=YES;
     USEDATE=YES;
     SCANTIME=YES;
RUN;


DATA work.branch_restrict06_correl(drop = year restriction_index);
  set work.branch_restrict06;
  rename state_ = state;
  restrict06 = input(restriction_index, 2.);
  label restrict06 = "Branching Restriction 2006";
run;
proc sort data = branch_restrict06_correl;
 by state;
run;


PROC IMPORT OUT= WORK.Branch_restrict86 
            DATAFILE= "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\brdumm2.xls" 
            DBMS=EXCEL REPLACE;
     RANGE="sasdata$"; 
     GETNAMES=YES;
     MIXED=NO;
     SCANTEXT=YES;
     USEDATE=YES;
     SCANTIME=YES;
RUN;

%let list = AL AK AZ AR CA CO CT DE FL GA HI
            ID IL IN IA KS KY LA ME MD MA MI 
            MN MS MO MT NE NV NH NJ NM NY NC 
            ND OH OK OR PA RI SC SD TN TX UT 
            VT VA WA WV WI WY ;

PROC TRANSPOSE DATA = branch_restrict86 OUT = branch_restrict86_panel PREFIX = restrict;
  ID quarter;
  VAR &list;
RUN;

DATA branch_restrict86_correl (keep = state restrict86) ;
  SET branch_restrict86_panel (keep = _name_ restrict1986_4) ;
  state = substr(_name_,1,2);
  IF restrict1986_4 = 'u' THEN restrict86 = 3;
   ELSE IF restrict1986_4 = 'l' THEN restrict86 = 2;
   ELSE IF restrict1986_4 = 's' THEN restrict86 = 1;
  label restrict86 = "Branching Restrictions, 1986Q4";
RUN;
proc sort data = branch_restrict86_correl;
 by state;
run;

/* Now we need to merge all of the data sets together, to get one panel data set of independent variables for correlation measurse */

LIBNAME fail "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures";

DATA fail.correlation_variables;
  MERGE hpi_correl unemp_correl all_correl num_correl pci_correl gdp_correl loan_correl bbq_correl branch_restrict06_correl branch_restrict86_correl;
  BY state;
RUN;


/* Lets also add the restriction index data to the map failure rate data (rates by state) to get a list of states by 
    failure rate, along with the number of failures, total number of banks, and value of the relevant branching index */
proc sort data = fail.map_panel;
  by state_cd;
run;
proc sort data = fail.fips_cd;
  by state_cd;
run;

DATA fail.branch_order;
  MERGE fail.map_panel fail.fips_cd;
  BY state_cd;
RUN;

proc sort data=fail.branch_order;
  by state_abr;
run;

DATA fail.branch_order (keep = state stnamebr failrt_86 failrt_06 nfail_1992 nfail_2010 totbnks_1986 totbnks_2006 restrict06 restrict86)  ;
  MERGE fail.branch_order (rename=(state_abr=state)) work.branch_restrict06_correl work.branch_restrict86_correl;
  BY state;
RUN;

proc sort data = fail.branch_order;
  by descending failrt_06;
run;

PROC EXPORT DATA= fail.branch_order 
            OUTFILE= "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\Failure Rates by Branch Restriction.xls" 
            DBMS=EXCEL REPLACE;
     SHEET="Data"; 
RUN;
