/* Program written by Craig Aubuchon in April 2010

   Create a panel data set of 1987-1992 and 2007-2010 failures.

   Create U.S. maps of failure rates by number of failures, and deposits (at the branch level).

   Create Density Plots of Banks by size.

   Note: For each period, we use the total number of banks(deposits) in the second quarter of the previous year -
    so 1986Q2 and 2006Q2.  We restrict our failure sample to the banks that failed during the subsequent period, 
    but were also in existence during the baseline year.  This means we lose 8 banks in the 2007-2010 period, and
    50 banks from the 1987-1992 period.

*/
 

OPTIONS LS = 75 PS = 60 NOCENTER PAGENO = 1 SYMBOLGEN MCOMPILENOTE = ALL MLOGIC;

/* A note on options: symbolgen specifies that log messages about macro variable references will be displayed
					  mcompilenote specifies that a note be completed for all macro compilations
					  mlogic specifies that messages about a macro action are printed to log during execution 
*/

LIBNAME fail            "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures";
 

/* Merge the two panel datasets together, to get one dataset of map data for all years */

DATA fail.map_panel;
  MERGE fail.map_panel1986 (rename=(rssd9210=state_cd)) fail.map_panel2006 (rename=(stnumbr=state_cd)) fail.fips_cd;
  BY state_cd;
RUN;

/* Now, make maps of the number of failures by state, and export to the Bank Failures Folder */

%MACRO yearloop(yearst=,yearend=,year=,);
  %DO i = 1 %TO 2;
  %LET yyst = %SCAN(&yearst,&i);
  %LET yyend = %SCAN(&yearend,&i);
  %LET yy = %SCAN(&year,&i);
    
/* Create a map of U.S. that illustrates number of failures by State */

data work.fail_sum;
  set fail.map_panel;
  state = state_cd;
  FORMAT state 2.0;
  /* can change ratio cut offs here */
 
%IF &yyend = 1992 %THEN %DO;
  %let c1 = 0;
  %let c2 = 10;
  %let c3 = 20;
  %let c4 = 50;
  %let c5 = 100;
%END;
 %ELSE %IF &yyend = 2010 %THEN %DO;
  %let c1 = 0;
  %let c2 = 1;
  %let c3 = 10;
  %let c4 = 20;
  %let c5 = 30;
 %END;

  if  &c1 =<nfail_&yyend< &c2          then sbgroup=1;
  if  &c2 =<nfail_&yyend< &c3          then sbgroup=2;
  if  &c3 =<nfail_&yyend< &c4          then sbgroup=3;
  if  &c4 =<nfail_&yyend< &c5          then sbgroup=4;
  if  &c5 =<nfail_&yyend               then sbgroup=5; 
  title;

run;
quit;

/* this changes the colors of the different ratios */

pattern1 value=msolid color=white;
pattern2 value=m4x45 color=vpab;
pattern3 value=msolid color=vpab;
pattern4 value=msolid color=vlib;
pattern5 value=msolid color=mob;
/* colors can be found in the SAS help manual under 'predefined SAS colors' */  

goptions ftext="Arial/bold" htext=.15 in;

**This creates the legend**;
legend1 label=none
       mode=share
	   down=5
	   offset=(-2.5 in /*the x axis */, .5 in /* y axis*/)
	         /*this offset value puts the legend in the bottom left hand corner */
 	   value=        (tick=1 justify=l "Bank Failures < &c2 "
			  tick=2 justify=l "&c2 =< Bank Failures < &c3"
			  tick=3 justify=l "&c3 =< Bank Failures < &c4"
			  tick=4 justify=l "&c4 =< Bank Failures < &c5"
              tick=5 justify=l "&c5 =< Bank Failures ");   

title1 height=2 " Bank Failures, &yyst to &yyend  ";
%IF &yyst = 1987 %THEN %DO;
title2 height=0.75 "Note: Includes only Chtr_type_code 200"; %END;
%ELSE %IF &yyst = 2007 %THEN %DO;
title2 height=0.75 "Note: Includes Chtr_type_code 200 300 310"; %END;

ODS pdf file = "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\2 Summary Map &yyend..pdf";
PROC GMAP MAP=maps.us DATA=work.fail_sum;
      ID state;
      CHORO sbgroup / coutline=megr legend=legend1;
	  
RUN;
ODS pdf close;
QUIT;

/* Maps of the failure rate (number of failures divided by total number of banks) by state                      */

/* Note: We're going to calculate a 'dynamic' range for the charts.  
          If SAS doesn't have a value in each 'bin' or 'sbgroup' then it won't create a correct chart.
          The distribution on the two failure rates is very different in the two periods, so the same cutoffs don't
          make sense.  We need unique cutoffs for each time period.                                             

    Alternatively, you could calculate the subgroups based on percentages.  Sort data by variable of interest,
     calculate a percentage rank (_n / nobs) and then define subgroups based on rank.  This would put an equal number of
     observations in each subgroup.                                                                              */

data work.fail_sum;
  set fail.map_panel;
  state = state_cd;
  FORMAT state 2.0;
  /* can change ratio cut offs here */

 %IF &yyend = 1992 %THEN %DO;
  %let c1 = 0;
  %let c2 = 1;
  %let c3 = 5;
  %let c4 = 10;
  %let c5 = 20;
%END;
 %ELSE %IF &yyend = 2010 %THEN %DO;
  %let c1 = 0;
  %let c2 = 1;
  %let c3 = 5;
  %let c4 = 10;
  %let c5 = 15;
 %END;

  if  &c1 =<failrt_&yy< &c2          then sbgroup=1;
  if  &c2 =<failrt_&yy< &c3          then sbgroup=2;
  if  &c3 =<failrt_&yy< &c4          then sbgroup=3;
  if  &c4 =<failrt_&yy< &c5          then sbgroup=4;
  if  &c5 =<failrt_&yy               then sbgroup=5; 
  title;
  
run;

/* this changes the colors of the different ratios */

pattern1 value=msolid color=white;
pattern2 value=m4x45 color=vpab;
pattern3 value=msolid color=vpab;
pattern4 value=msolid color=vlib;
pattern5 value=msolid color=mob;
/* colors can be found in the SAS help manual under 'predefined SAS colors' */  

goptions ftext="Arial/bold" htext=.16 in;

**This creates the legend**;
legend1 label=none
       mode=share
	   down=5
	   offset=(-2.5 in /*the x axis */, .45 in /* y axis*/)
	         /*this offset value puts the legend in the bottom left hand corner */
 	   value=        (tick=1 justify=l " Bank Failure Rate < &c2.% "
			  tick=2 justify=l "&c2.% =< Bank Failure Rate < &c3.%"
			  tick=3 justify=l "&c3.% =< Bank Failure Rate < &c4.%"
			  tick=4 justify=l "&c4.% =< Bank Failure Rate < &c5.%"
              tick=5 justify=l "&c5.% =< Bank Failure Rate ");   

title1 height=2.5 "Fig. 6: Bank Failure Rates in percent, &yyst to &yyend ";
/* %IF &yyst = 1987 %THEN %DO;
title2 height=0.75 "Note: Includes only Chtr_type_code 200"; %END;
%ELSE %IF &yyst = 2007 %THEN %DO;
title2 height=0.75 "Note: Includes Chtr_type_code 200 300 310"; %END; */

ODS pdf file = "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\2 Summary Map Rates &yyend..pdf";
PROC GMAP MAP=maps.us DATA=work.fail_sum;
      ID state;
      CHORO sbgroup / coutline=megr legend=legend1;
	  
RUN;
ODS pdf close;
QUIT;


/* Create a map of U.S. that illustrates failures as a percent of deposits, by state */

data work.fail_sum;
  set fail.map_panel;
  state = state_cd;
  FORMAT state 2.0;
 
  %let c1 = 0;
  %let c2 = 1;
  %let c3 = 5;
  %let c4 = 10;
  %let c5 = 20;
 
  if  &c1 =<dep_failrt&yy< &c2          then sbgroup=1;
  if  &c2 =<dep_failrt&yy< &c3          then sbgroup=2;
  if  &c3 =<dep_failrt&yy< &c4          then sbgroup=3;
  if  &c4 =<dep_failrt&yy< &c5          then sbgroup=4;
  if  &c5 =<dep_failrt&yy               then sbgroup=5; 
  title;

run;
quit;

/* this changes the colors of the different ratios */

pattern1 value=msolid color=white;
pattern2 value=m4x45 color=vpab;
pattern3 value=msolid color=vpab;
pattern4 value=msolid color=vlib;
pattern5 value=msolid color=mob;
/* colors can be found in the SAS help manual under 'predefined SAS colors' */  

goptions ftext="Arial/bold" htext=.15 in;

**This creates the legend**;
legend1 label=none
       mode=share
	   down=5
	   offset=(-2.5 in /*the x axis */, .45 in /* y axis*/)
	         /*this offset value puts the legend in the bottom left hand corner */
 	   value=        (tick=1 justify=l "Ratio of Deposits < &c2.% "
			  tick=2 justify=l "&c2.% =< Ratio of Deposits < &c3.%"
			  tick=3 justify=l "&c3.% =< Ratio of Deposits < &c4.%"
			  tick=4 justify=l "&c4.% =< Ratio of Deposits < &c5.%"
              tick=5 justify=l "&c5.% =< Ratio of Deposits ");   

title1 height=2.5 "Fig. 7: Bank Failure Ratio by Branch Deposits, &yyst to &yyend";
/*%IF &yyst = 1987 %THEN %DO;
title2 height=0.75 "Note: Includes only Chtr_type_code 200"; %END;
%ELSE %IF &yyst = 2007 %THEN %DO;
title2 height=0.75 "Note: Includes Chtr_type_code 200 300 310"; %END; */

ODS pdf file = "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\2 Dep_failrt Map &&yyend..pdf";
PROC GMAP MAP=maps.us DATA=work.fail_sum;
      ID state;
      CHORO sbgroup / coutline=megr legend=legend1;
	  
RUN;
ODS pdf close;
QUIT;

%END;%MEND;%yearloop(yearst=1987 2007,yearend=1992 2010,year=86 06);


PROC EXPORT DATA= fail.map_panel 
            OUTFILE= "C:\h1cpa01\work\Current Wheelock\Active\Review Bank Fail\Failures_Summary.xls" 
            DBMS=EXCEL REPLACE;
     SHEET="summary"; 
     NEWFILE=YES;
RUN;
