/* Code written by Craig Aubuchon March 2010 
   
   Create a panel data set of failures against all banks, in the year 2006 baseline.

   This file creates two different data files:
 
     'map_panel2006' contains data on the number of failures by state (banks that failed from 2007 to 2010Q1 -
       but were in existence in the 2006 base year), a failure rate (number of failures divided by number of banks per state),
       and deposit failure rate - which is the amount of deposits calculated at the branch level of failed banks divided by
       the sum of all deposits in branches, by state.

      To do this, I first calculate the deposit data, since this requires the full SOD data at the branching level.
    
      Number of Failures and failure rate data is calculated on a smaller data set (of only bank main offices).

      'panel2006' contains the data for bank main offices, including total assets.  This is the data that 
        is used to calculate descriptive statistics and also density plots.


*/


OPTIONS LS = 75 PS = 60 NOCENTER PAGENO = 1 SYMBOLGEN MCOMPILENOTE = ALL MLOGIC;

/* A note on options: symbolgen specifies that log messages about macro variable references will be displayed
					  mcompilenote specifies that a note be completed for all macro compilations
					  mlogic specifies that messages about a macro action are printed to log during execution 
*/

LIBNAME raw_SOD         "C:\h1cpa01\work\Current Wheelock\Active\FourFirmConcentration\SOD_raw_data";
LIBNAME fail            "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures";

/* Fail.chtrcd_sod contains data from the NIC attributes table */
PROC contents data = fail.chtrcd_sod;
RUN;

/* raw_sod.sodmic06 contains the full data set from the 2006 SOD */

PROC contents data = raw_SOD.sodmic06;
RUN;

/* Calculate branching data from 2006 SOD data */

PROC SQL;

CREATE TABLE fail.sod2006 AS

SELECT sod.rssdid, sod.name, sod.city, sod.stname, sod.cert, sod.bkbr, sod.bkmo, sod.uninumbr, sod.stnumbr, sod.cntynumb, sod.zipbr, sod.asset, sod.depdom,
       sod.hctmult, sod.hctone, sod.hctnone, sod.specgrp, sod.specdesc, sod.depsumbr, 
       count(sod.rssdid)  as totbr,
       count(distinct sod.cntynumb) as tot_cntybr, 
       count(distinct sod.zipbr) as tot_zipbr, 
       count(distinct sod.stnumbr) as tot_statebr 
        	  	   
FROM raw_SOD.sodmic06 as sod

GROUP BY rssdid 
ORDER BY rssdid;

QUIT;

/* Now I need to merge charter type code(and bnk type analysis code) to the SOD data */
/* We expect to drop a few SOD observations.  There are 8767 banks in the full 2006 sample */

DATA sod2006_panel;
  MERGE fail.sod2006 (IN = sod_restrict rename = (rssdid = id_rssd)) fail.chtrcd_sod; 
  BY id_rssd;
  IF sod_restrict; 
RUN;

data test;
  set sod2006_panel;
  where bkmo = 1 and bnk_type_analys_cd = 0;
run;

/* Restrict sample to banks with analysis code = 0 */
DATA sod2006_panel;
  SET sod2006_panel;
  WHERE bnk_type_analys_cd = 0;
RUN;

/******************************************************************************************************************/
/* Now, I need to add in a dummy code for the banks that will fail from 2007 to 2010 - 
    We need to use the original panel of bank failures (from 'panel_failures.sas') called fail.failures_2010.

   We want to get a set of summary statistics for all banks that failed using the same financial data (2006) as the
    comparison group.

   We also want to output a panel data set of failures by state code, so that we can make maps.  This also entails
    calculating deposits in failed branches, for the deposit failure rate. 

   Note: We lose 8 'failures'.  Sample size is cut from 206 to 198.  This means 8 failed banks are in the later SOD data
         but not in the 2006 data.                                                          
/*******************************************************************************************************************/

DATA work.sod2006_panel;
  MERGE work.sod2006_panel (IN=restrict) fail.failures_2010 ;
  BY id_rssd;
  IF restrict;
  IF fail ~=1 THEN fail = 0;
RUN;

/* Pool 300 and 310 together as 'savings institutions */

DATA work.sod2006_panel;
  SET work.sod2006_panel;
  IF chtr_type_cd > 200 THEN save = 1;
   ELSE IF chtr_type_cd = 200 THEN save = 0;
  LABEL save = "savings institution";
RUN;

/* First, calculate a deposit failure rate (since we data at the branch level for this */

PROC SQL; 
  CREATE TABLE sod2006_panel2 as
  SELECT *, sum(depsumbr) as tot_dep06_st, 
            sum(case when fail = 1 then depsumbr else 0 end) as tot_fail_dep06_st
  FROM sod2006_panel
   WHERE stnumbr NOT IN(11,60,64,66,68,69,70,72,74,78) 
  GROUP BY stnumbr;
QUIT;

/* Lets look and see who the biggest failing branches are by state */
proc sort data = sod2006_panel2;
  by stnumbr descending depsumbr ;
run;
proc print data = sod2006_panel2 (obs = 20);
  title1 'Failures by Branch in Illinois';
  where stnumbr = 17;
  var id_rssd name stnumbr asset depdom chtr_type_cd totbr state_cd fail;
run;
proc print data = sod2006_panel2 (obs = 30);
  title1 'Failures by Branch in New York';
  where stnumbr = 36 ;
  var id_rssd name stnumbr asset depdom depsumbr chtr_type_cd totbr state_cd fail;
run;

proc print data = sod2006_panel2;
  title1 'Bank failures in New York';
  where fail = 1 and state_cd = 36;
run;
title1;

proc sql;
  create table newyork_depfail as 
  select *, sum(depsumbr) as branch_deposits, count(depsumbr) as branches
  from sod2006_panel2
  where stnumbr = 36 and fail = 1
  group by id_rssd;
quit;

proc print data = newyork_depfail;
run;

/* Make sure to sort data by stnumbr so that the first.stnumbr code below works */
proc sort data = sod2006_panel2;
  by stnumbr;
run;

/* Now, I can keep one observation per bank for the panel2006 data set; And I can keep only the summary variables for each state for the map data */

DATA map_panel2006 (keep = stnumbr tot_dep06_st tot_fail_dep06_st);
  SET sod2006_panel2;
  IF first.stnumbr THEN OUTPUT;
  BY stnumbr;
RUN;

DATA map_panel2006;
  SET map_panel2006;
  dep_failrt06 = ( tot_fail_dep06_st / tot_dep06_st ) * 100 ;
  FORMAT dep_failrt06 3.2;
RUN;

/* Now I need to get counts on the number of banks and the number of failures for each state */

DATA sod2006_panel3;
  SET sod2006_panel2;
  WHERE BKMO = 1;
RUN;

proc means data = sod2006_panel3;
  class fail;
run;

PROC SQL; 
  CREATE TABLE sod2006_panel4 as
  SELECT *, count(id_rssd) as totbnks_2006, 
            sum(case when fail = 1 then fail else 0 end) as nfail_2010
  FROM sod2006_panel3
  GROUP BY stnumbr;
QUIT;

DATA map_panel20062 (keep = stnumbr nfail_2010 totbnks_2006);
  SET sod2006_panel4;
  IF first.stnumbr THEN OUTPUT;
  BY stnumbr;
RUN;

DATA map_panel20062;
  SET map_panel20062;
  failrt_06 = ( nfail_2010 / totbnks_2006 ) * 100 ;
  FORMAT failrt_06 3.2;
RUN;

/* Now we have both types of data - deposit failure rates (by branch) and number of banks, number of failures, by state */
/*  We'll merge them together into one panel for the map data */

DATA fail.map_panel2006;
  MERGE map_panel2006 map_panel20062;
  BY stnumbr;
RUN;

proc sort data = fail.map_panel2006;
  by descending failrt_06;
run;

proc print data = fail.map_panel2006;
run;
/* And create the year 2006 panel data for summary statistics and density plots */

DATA fail.panel2006;
  SET sod2006_panel4;
RUN;

/* Calculate the percentage of deposits held by failed banks to all banks */

PROC SQL;
  SELECT fail, depdom, sum(depdom) as tot_dep
  FROM fail.panel2006
  GROUP BY fail;
QUIT;

PROC PRINT data = fail.panel2006;
  VAR nm_short asset depdom;
  where fail = 1 and bhc_ind = 2;
RUN;

/* Find total deposits for WaMu */

PROC SORT data = fail.panel2006;
  by descending fail descending asset;
run;
proc print data = fail.panel2006 (obs = 20);
  var name  city stname depdom totbr tot_statebr entity_type;
run;
 
DATA fail.top20fail (keep = name city stname asset depdom totbr tot_statebr entity_type chtr_type_cd);
  SET fail.panel2006 (obs = 20);
  label name = "Bank Name";
  label depdom = "Total Deposits in Thousands of Dollars";
  label totbr = "Total Number of Branches";
  label tot_statebr = "Total Number of States with a Branch";
  label entity_type = "Entity Type";
RUN;

/*
PROC EXPORT DATA= FAIL.TOP20FAIL 
            OUTFILE= "C:\h1cpa01\work\Current Wheelock\Active\Bank Failures\Top 20 Bank Failures.xls" 
            DBMS=EXCEL REPLACE;
     SHEET="table2"; 
RUN;
*/


